classdef ActiveBasisLocalNormSVM
    
    % ---------------------------------------------------------------------
    % Supported functionalities:
    %   
    %   1. Learning active basis/texture templates category by category.
    %
    %   2. Cross-validation and testing category by category.
    %
    % 
    % Future work:
    %
    %   1. Use a standardized image pyramid and a better modularized
    %       convolution method.
    %
    % 
    % ---------------------------------------------------------------------
    
    % member variables
    properties
        categories;	% collection of category names
        iRun;		% index for cross validation run
        iCategory;	% index of current category
        category;	% name of current category
        OverwriteModeIsOn = true;
        className = 'ActiveBasisLocalNormSVM';
    end

    properties
        maxImArea = 1000*1000;	% maximum allowed image area
        posFolder;		% folder for positive example images
        negFolder = 'Image/Natural';	% folder for random natural images
        featureVersion = 'GaborS1';		% feature name
        S1FeatureType = 'S1LocalNorm';
        windowSize = 150; %
        Totalsketch = 200; % maximum allowed number of features
        epsilon = .1; % allowed correlation between selected Gabors
        sub = 1;    % subsampling rate for Gabor
        Lrange = 6; % allowed shifting in location (times sub pixels)
        Orange = 1; % allowed shifting in orientation
        norient = 16; % number of Gabor orientations
        %%% Note: Lrange*sub is the pixels allowed to shift along normal
        %%%       Lrange*sub should be the same in learning and testing
        SHUTUP = 0.; % SHUTUP is to neglect the positions with low average responses
        ToFindWhileLearning = true; % use EM-type iterations for learning templates for objects with unknown locations
        sizeGabor = 17;
        scale = 0.7;
        gainThres = 0.2;
		test_gainThres = 0.2;
        max_EM_iter = 10;
        displayModeIsOn = false;
        svm_overwrite_example_file = false;
    end
    
    properties
        ap;				% average precision
        testAUC;		% area under ROC curve
        tp;				% true positive rates
        fp;				% false positive rates
        rec;			% recall rates
        prec;			% precision rates
        EER;			% equal-error rate
    end
    
    % functions
    methods
        
        % constructor
        function obj = ActiveBasisLocalNormSVM(iRun, iCat, categories)
            obj.iRun = iRun;
            obj.iCategory = iCat;
            obj.categories = categories;
            obj.category = categories{iCat};
        end
        
        

        %% learns a single template from a set of images (positive examples)
        function learn(this)
            
            disp(sprintf('start learning: %d %s run %d',this.iCategory,this.category,this.iRun));
            close all
            if this.displayModeIsOn
                figure;
                imshow( zeros(this.windowSize,this.windowSize),[] );
            end
            
			load('Feature/naturalStat_GaborS1_LocalNorm.mat'); % load natural statistics
            % TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % ==================================================================
            %  input images from a folder
            % ==================================================================
            file0 = sprintf('Split/%s/split%d.mat',this.category,this.iRun);
            load(file0,'posTrainImages');
            nimage = length(posTrainImages);
            
            % prepare Gabor filters and compute the inhibition map
            tic
            [allfilter, allsymbol] = makefilter(this.scale, this.norient);  % generate Gabor filters
            for o = 1:this.norient
                allsymbol{o} = single(allsymbol{o});
            end
            h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
            C = corr(allfilter, this.epsilon);    % generate the inhibition maps
            for o = 1:this.norient
                for o2 = 1:this.norient
                    C{o,o2} = single(C{o,o2});
                end
            end
            
            % load and inscribe images into the template lattice
            SUM1mapLearn = cell(nimage,this.norient);
            stored_SUM1map = cell(nimage,this.norient);
            for i = 1:nimage
                load(sprintf('Feature/%s/%s/%s.mat',...
                    this.featureVersion,this.category,posTrainImages{i}),...
                    'S1LocalNorm','RANGE');
				S1LocalNorm = convert2float(S1LocalNorm,RANGE);
                % inscribe the S1 map into a square bounding box
                for o = 1:this.norient
                    SUM1mapLearn{i,o} = zeros(this.windowSize, this.windowSize,'single');
                    tmp = inscribe([this.windowSize this.windowSize],S1LocalNorm{o},0);
                    SUM1mapLearn{i,o}(:) = tmp(:);
                end
            end
			
            for i = 1:nimage
            	for o = 1:this.norient
					stored_SUM1map{i,o} = zeros(size(SUM1mapLearn{i,o}),'single');
					stored_SUM1map{i,o}(:) = SUM1mapLearn{i,o}(:);
            	end
            end
            
			SUM1LocalMeanLearn = cell(nimage,this.norient);
			stored_SUM1LocalMean = cell(nimage,this.norient);
            for i = 1:nimage
                load(sprintf('Feature/%s/%s/%s.mat',...
                    this.featureVersion,this.category,posTrainImages{i}),'S1LocalMean','localHalfx','RANGE');
				S1LocalMean = convert2float(S1LocalMean,RANGE);
                % inscribe the S1 local mean map into a square bounding box
                for o = 1:this.norient
                    SUM1LocalMeanLearn{i,o} = zeros(this.windowSize, this.windowSize,'single');
                    tmp = inscribe([this.windowSize this.windowSize],S1LocalMean{o},0);
                    SUM1LocalMeanLearn{i,o}(:) = tmp(:);
                end
            end
			
			for i = 1:nimage
            	for o = 1:this.norient
					stored_SUM1LocalMean{i,o} = zeros(size(SUM1LocalMeanLearn{i,o}),'single');
					stored_SUM1LocalMean{i,o}(:) = SUM1LocalMeanLearn{i,o}(:);
            	end
            end
			
            % -----------------------------------------------
            % begin EM-type iterations:
            % -----------------------------------------------
			if this.ToFindWhileLearning
				max_iter = this.max_EM_iter;
			else
				max_iter = 1;
            end
			
            diff_in_detection = 1; % the proportion that the current detection is different from last time
            current_row_translation = zeros(nimage,1);
            current_col_translation = zeros(nimage,1);
			for iter = 1:max_iter
			
				disp( sprintf('begin iteration %d', iter) );
				% ---- M step ----
				%
				% learning of sketch
				%
				tic
				load('Feature/naturalStat_GaborS1_LocalNorm.mat'); % load natural statistics
		        Mi = zeros(1, this.Totalsketch, 'single');  % orientation of selected Gabor
		        Mx = zeros(1, this.Totalsketch, 'single');  % position of selected Gabor
		        My = zeros(1, this.Totalsketch, 'single'); 
		        Mm = zeros(1, this.Totalsketch, 'single');  % lambda
		        Mm1 = zeros(1, this.Totalsketch, 'single');  % logZ
		        gain = zeros(1, this.Totalsketch, 'single'); % information gains (bits)
		        sym = zeros(this.windowSize,this.windowSize, 'single');  % symbolic plot of selected Gabor
		        Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
		        for i = 1 : nimage
		            Asym{i} = zeros(size(SUM1mapLearn{i,1}),'single');  
		        end
		        tss = zeros(nimage, 1, 'single');  % total scores for testing images
		        gain = zeros(this.Totalsketch, 1, 'single');  % coding gain for selected Gabor
		        binnum = 50; % number of bins
		        subsampleM1 = 1;
		        MAX1mapLearn = cell(size(SUM1mapLearn));
  				ARGMAX1mapLearn = cell(size(SUM1mapLearn));
		        for i = 1:nimage
				    [MAX1mapLearn(i,:) ARGMAX1mapLearn(i,:) M1RowShift M1ColShift M1OriShifted] = ...
				    	mexc_ComputeMAX1( this.norient, SUM1mapLearn(i,:), single(this.Lrange*this.sub)/(h*2+1),...
		            		this.Orange, single(h*2+1), subsampleM1 );
		        end

		        mexc_SharedSketch(this.norient, this.Lrange * this.sub, this.Orange, subsampleM1, ... % about active basis  
				   this.Totalsketch, nimage, this.windowSize, this.windowSize, ...
				   SUM1mapLearn, MAX1mapLearn, ARGMAX1mapLearn, ... % about training images 
				   h, C, allsymbol(1, :), ... % about filters
				   single(M), single(lam), single(e), single(lz), ... % about exponential model 
				   Mi, Mx, My, Mm, Mm1,gain, ... % learned parameters
				   sym, Asym,... % learned templates 
				   M1RowShift, M1ColShift, M1OriShifted);
		        
		        nF = max(1,sum(gain>this.gainThres));
		        Mi = Mi(1:nF);
		        Mx = Mx(1:nF);
		        My = My(1:nF);
		        Mm = Mm(1:nF);
		        Mm1 = Mm1(1:nF);
		        sym = displayGaborTemplate([this.windowSize this.windowSize], Mx,My,Mi,Mm);
		        
		        if ~exist(sprintf('Result/%s/%s/',this.category,this.className),'dir')
		            mkdir(sprintf('Result/%s/%s/',this.category,this.className));
		        end
		        
		        Lrange = this.Lrange; Orange = this.Orange; sub = this.sub;
		        S2Template = struct( 'selectedRow', single(Mx)-1-floor(this.windowSize/2), ...
		        	'selectedCol', single(My)-1-floor(this.windowSize/2), ...
		        	'selectedOri', single(Mi), ...
		        	'selectedScale', zeros(1,length(Mx),'single'), ... 
		        	'selectedLambda', single(Mm), ...
		        	'selectedLogZ', single(Mm1), ...
		        	'subsampleM1', 1, ...
					'selectedGain', gain,...
		        	'sym', sym ...
		        	);
		        
		        save(sprintf('Result/%s/%s/model_run%d.mat',this.category,this.className,this.iRun),...
		            'Mi','Mx','My', 'Mm','Mm1','C','h','scale',...
		            'norient','allfilter','allsymbol','Asym','Upperbound',...
		            'M','lam','e', 'lz', 'sym','tss','gain','Lrange','Orange','sub','S2Template');
		        disp(['learning time for sketch: ' num2str(toc) ' seconds']);
				
				%
				% learning of texture
				%
				tic
				load('Feature/naturalStat_GaborS1_LocalMean.mat'); % load natural statistics
				storedMean = e;
				storedLambda = lam;
				storedLogZ = lz;
				templateSize = size(SUM1LocalMeanLearn{1});
				meanLocalMeanMap = zeros(size(SUM1LocalMeanLearn{1},1), size(SUM1LocalMeanLearn{1},2),length(SUM1LocalMeanLearn));
				for i = 1:nimage
					for o = 1:norient
						meanLocalMeanMap(:,:,o) = meanLocalMeanMap(:,:,o) + 1/nimage * SUM1LocalMeanLearn{i,o};
					end
				end
				
				maxNumFeature = this.Totalsketch;
				Mi = zeros(1, maxNumFeature);  % orientation of selected Gabor
				Mx = zeros(1, maxNumFeature);  % position of selected Gabor
				My = zeros(1, maxNumFeature); 
				Mm = zeros(1, maxNumFeature);  % mean
				lambda = zeros(1, maxNumFeature);  % lambda
				logZ = zeros(1, maxNumFeature);  % logZ
				gain = zeros(maxNumFeature, 1);  % coding gain for selected Gabor

				gainMap = zeros(templateSize(1),templateSize(2),this.norient);
				ind = zeros(templateSize(1),templateSize(2),o);
				ax = localHalfx;
				for x = (h+ax)+1:templateSize(1)-(h+ax)
					for y = 1+(h+ax):templateSize(2)-(h+ax)
						for o = 1:this.norient
							[dif ind(x,y,o)] = min( abs(storedMean - meanLocalMeanMap(x,y,o)) );
							ind(x,y,o) = min(length(storedMean),max(1,ind(x,y,o)));
							gainMap(x,y,o) = storedMean(ind(x,y,o)) * storedLambda(ind(x,y,o)) - storedLogZ(ind(x,y,o));
						end
					end
				end
				inhibited = zeros(templateSize(1),templateSize(2),this.norient);

				nF = maxNumFeature;
				for k = 1:nF
					[maxGain here] = max(gainMap(:) .* double(1-inhibited(:))  );
					binNo = ind(here);

					if maxGain <= this.gainThres
						nF = k-1;
						break
					end

					ori = ceil(here/templateSize(1)/templateSize(2)); % start from 1 !!
					Mi(k) = ori;
					col = ceil((here - (ori-1) * templateSize(1) * templateSize(2))/templateSize(1));
					My(k) = col;
					row = here - (ori-1) * templateSize(1) * templateSize(2) - (col-1)*templateSize(1); 
					Mx(k) = row;
					
					gain(k) = maxGain;
					Mm(k) = storedMean(binNo);
					lambda(k) = storedLambda(binNo);
					logZ(k) = storedLogZ(binNo);

					% do inhibition
					left = max(1,col-ax-h);
					right = min(templateSize(2),col+ax+h);
					top = max(1,row-ax-h);
					bottom = min(templateSize(1),row+ax+h);
					for iOri = ori-1:ori+1
						iOri2 = iOri;
						if iOri < 1
							iOri2 = iOri + this.norient;
						end
						if iOri > this.norient
							iOri2 = iOri - this.norient;
						end
						inhibited(top:bottom,left:right,iOri2) = 1;
						gainMap(top:bottom,left:right,iOri2) = 0;
					end
				end
				Mm = Mm(1:nF); Mx = Mx(1:nF); My = My(1:nF);
				gain = gain(1:nF); Mi = Mi(1:nF);
				lambda = lambda(1:nF);
				logZ = logZ(1:nF);
				
				S2TemplateTexture = struct( 'selectedRow', single(Mx)-1-floor(this.windowSize/2), ...
		        	'selectedCol', single(My)-1-floor(this.windowSize/2), ...
		        	'selectedOri', single(Mi)-1, ...
		        	'selectedScale', zeros(1,length(Mx),'single'), ... 
		        	'selectedLambda', single(lambda), ...
		        	'selectedLogZ', single(logZ), ...
		        	'subsampleM1', 1, ...
					'selectedGain', gain,...
		        	'sym', sym ...
		        	);
                
                if this.displayModeIsOn
                    imshow( sym, [] );
                    pause(0.2);
                end
				
		        save(sprintf('Result/%s/%s/model_run%d.mat',this.category,this.className,this.iRun),...
		            '-append', 'S2TemplateTexture');
				disp(['learning time for texture: ' num2str(toc) ' seconds']);
				
		        % ---- E step ----
		        % prepare [SUM1mapLearn] for the next learning step
		        if iter >= max_iter
		        	break;
		        end

				% - obtain transformed version of templates
				% do nothing; (no rotation, no scaling, only translation)
				
				for iImage = 1:size(SUM1mapLearn,1)
				
					% - compute MAX1 maps
					locationPerturbationFraction = this.Lrange * sub / double(this.sizeGabor);
					orientationPerturbationRadius = this.Orange;
					subsampleM1 = 1;
					M1Map = mexc_ComputeMAX1( this.norient, stored_SUM1map(iImage,:), locationPerturbationFraction, ...
						orientationPerturbationRadius, this.sizeGabor, subsampleM1 );
				
					% - compute SUM2 maps of transformed templates on original feature maps		
					subsampleS2 = 1;

					S2Map = mexc_ComputeSUM2( this.norient, M1Map, {S2Template}, subsampleS2 );
					
					% - compute SUM2 maps for texture
					
					if ~isempty(S2TemplateTexture.selectedRow)
						S2MapTexture = mexc_ComputeSUM2( this.norient, stored_SUM1LocalMean(iImage,:), {S2TemplateTexture}, subsampleS2 );
					else
						S2MapTexture = cell(1,numel(S2Map));
						S2MapTexture{ii} = single( zeros(size(S2Map{ii})) );
					end
				
					for ii = 1:numel(S2Map)
						S2Map{ii} = S2Map{ii} + S2MapTexture{ii};
					end

					% - find the best match (global max)																						
					[Max2 bestInd] = max(S2Map{1}(:));
                    
					col_translation = ceil( bestInd / this.windowSize );
					row_translation = bestInd - (col_translation-1) * this.windowSize;
					col_translation = col_translation - 1;
					row_translation = row_translation - 1;

					% - crop SUM1 (sigmoid saturated) maps	
					denseX = -floor(this.windowSize/2) + (1:this.windowSize)-1;
					denseY = denseX;
					count = 0;
					inRow = zeros(length(denseX)*length(denseY),1,'single');
					inCol = zeros(length(denseX)*length(denseY),1,'single');
					inOri = zeros(length(denseX)*length(denseY),1,'single');
					inScale = zeros(length(denseX)*length(denseY),1,'single');
					for y = denseY
						for x = denseX
						    count = count+1;
						    inRow(count) = x;
						    inCol(count) = y;
						end
					end

					% use mex-C code to transform the dense grid, which is used to fetch SUM1 map pixels
					[outX outY outOri] = mexc_TemplateAffineTransform( 0, 2^0, 2^0, 0, inRow, inCol, inOri, inScale, this.norient );
					
					tScale = 0; destHeight = this.windowSize; destWidth = this.windowSize; nScale = 1;
					SUM1mapLearn(iImage,:) = mexc_CropInstance( stored_SUM1map(iImage,:),...
				    	row_translation,...
				    	col_translation,...
				    	0,tScale,outX,outY,destWidth,destHeight,this.norient,1 );

					SUM1LocalmeanLearn(iImage,:) = mexc_CropInstance( stored_SUM1LocalMean(iImage,:),...
				    	row_translation,...
				    	col_translation,...
				    	0,tScale,outX,outY,destWidth,destHeight,this.norient,1 );
                    
                    current_row_translation(iImage) = row_translation;
                    current_col_translation(iImage) = col_translation;
                    
                    %{
                    figure;
                    subplot(1,4,1); imshow(S2Map{1},[]);
                    subplot(1,4,2); imshow(stored_SUM1map{iImage,1},[]);
                    subplot(1,4,3); imshow(SUM1mapLearn{iImage,1},[]);
                    subplot(1,4,4); imshow(sym,[]);
                    title(sprintf('%d,%d',row_translation,col_translation));
                    pause;
                    close
                    %}
                end
                
                % compute the difference in detection
                nImage = size(SUM1mapLearn,1);
                if iter == 1
                    diff_in_detection = 1;
                else
                    count_dif = 0;
                    for iImage = 1:nImage
                        if abs( last_row_translation(iImage) - current_row_translation(iImage) ) > 3 ...
                            || abs( last_col_translation(iImage) - current_col_translation(iImage)) > 3
                            count_dif = count_dif + 1;
                        end
                    end
                    diff_in_detection = count_dif / nImage;
                end
                disp(sprintf('difference in detection: %.3f',diff_in_detection));
                last_row_translation = current_row_translation;
                last_col_translation = current_col_translation;
                
                if diff_in_detection < 0.2
                    break;
                end
                
            end

	        S2Template.extraLogZ = 0; %log(extraZ);
			
			save(sprintf('Result/%s/%s/model_run%d.mat',this.category,this.className,this.iRun),...
		            '-append','S2Template');
        end
        
        
        % -----------------------------------------------------
        %% prepare input for 1-vs-all svm learners
        % -----------------------------------------------------
        function prepare_svmtrain_input(this)
            nCat = length(this.categories);
            for iCat = 1:nCat
                category = this.categories{iCat};
        		load(sprintf('Result/%s/%s/model_run%d.mat',category,this.className,this.iRun),'S2Template','h','S2TemplateTexture');
				ind = find(S2Template.selectedGain > this.test_gainThres);
				S2Template.selectedRow = S2Template.selectedRow(ind);
				S2Template.selectedCol = S2Template.selectedCol(ind);
				S2Template.selectedOri = S2Template.selectedOri(ind);
				S2Template.selectedScale = S2Template.selectedScale(ind);
				S2Template.selectedLambda = S2Template.selectedLambda(ind);
				S2Template.selectedLogZ = S2Template.selectedLogZ(ind);
        		list_of_S2Template{iCat} = S2Template;
                
                ind = find(S2TemplateTexture.selectedGain > this.test_gainThres);
				S2TemplateTexture.selectedRow = S2TemplateTexture.selectedRow(ind);
				S2TemplateTexture.selectedCol = S2TemplateTexture.selectedCol(ind);
				S2TemplateTexture.selectedOri = S2TemplateTexture.selectedOri(ind);
				S2TemplateTexture.selectedScale = S2TemplateTexture.selectedScale(ind);
				S2TemplateTexture.selectedLambda = S2TemplateTexture.selectedLambda(ind);
				S2TemplateTexture.selectedLogZ = S2TemplateTexture.selectedLogZ(ind);
        		list_of_S2TemplateTexture{iCat} = S2TemplateTexture;
            end
            disp('templates loaded');

            for iCat = 1:nCat
                outputfilepath = sprintf( 'Result/%s/%s/svmtrain_run%d.txt', this.categories{iCat}, this.className, this.iRun );
                fid = fopen(outputfilepath,'w');
                fclose(fid);
            end
            clear iCat; clear S2Template; clear S2TemplateTexture
            
            for iCategory = 1:nCat
                tic
        		fprintf(1,'extracting selected features for category %s: ',this.categories{iCategory});
                
		    	file0 = sprintf('Split/%s/split%d.mat',this.categories{iCategory},this.iRun);
		        load(file0,'posTrainImages');
		        nimage = length(posTrainImages);

		        for i = 1:nimage
		        	for kk = 1:length(num2str(i-1))
		        		if i == 1
		        		else
		        			fprintf('\b');
		        		end
		        	end
		        	fprintf(1,'%d',i);
				    % for each image, test all templates on it
				    load(sprintf('Feature/%s/%s/%s.mat',...
		                this.featureVersion,this.categories{iCategory},posTrainImages{i}),...
						'S1LocalMean','M1LocalNorm','RANGE');

					S1LocalMean = convert2float(S1LocalMean,RANGE);
					M1 = convert2float(M1LocalNorm,RANGE);
                    
		            % - compute SUM2 maps
					
                    subsampleS2 = 5;
                    S2MapSketch = mexc_ComputeSUM2( this.norient, M1, list_of_S2Template, subsampleS2 );
                    S2MapTexture = mexc_ComputeSUM2( this.norient, S1LocalMean, list_of_S2TemplateTexture, subsampleS2 );
                    S2Map = cell(size(S2MapSketch));
                    for ii = 1:numel(S2Map)
                        S2Map{ii} = S2MapSketch{ii} + S2MapTexture{ii};
                    end
				    
                    for jT = 1:nCat
                        S2Template = list_of_S2Template{jT};
                        S2TemplateTexture = list_of_S2TemplateTexture{jT};
                        outputfilepath = sprintf( 'Result/%s/%s/svmtrain_run%d.txt', this.categories{jT}, this.className, this.iRun );
                        fid = fopen(outputfilepath,'a');
                        
                        [Max2 bestInd] = max(S2Map{jT}(:));
                        col_translation = ceil( bestInd / size(S2Map{1},1) );
                        row_translation = bestInd - (col_translation-1) * size(S2Map{1},1);
                        
                        col_translation = (col_translation - 1) * subsampleS2;
                        row_translation = (row_translation - 1) * subsampleS2;

                        % output ground truth
                        if jT == iCategory
                            fprintf(fid, '%d ', 1);
                        else
                            fprintf(fid, '%d ', -1);
                        end

                        % output sketch features
                        responses_sketch = mexc_ExtractSelectedFeatureResponse( this.norient, M1, list_of_S2Template, subsampleS2, S2Map, outputfilepath );
                        responses_tex = mexc_ExtractSelectedFeatureResponse( this.norient, S1LocalMean, list_of_S2TemplateTexture, subsampleS2, S2Map, outputfilepath );
                        
                        count = 1;
                        
                        S2score = 0;
                        for jF = 1:length(S2Template.selectedRow)
                            val = responses_sketch{jT}(jF);
                            fprintf(fid, '%d:%.5f ', count, val );
                            S2score = S2score + val * S2Template.selectedLambda(jF) - S2Template.selectedLogZ(jF);
                            count = count + 1;
                        end

                        
                        for jF = 1:length(S2TemplateTexture.selectedRow)
                            val = responses_tex{jT}(jF);
                            fprintf(fid, '%d:%.5f ', count, val );
                            S2score = S2score + val * S2TemplateTexture.selectedLambda(jF) - S2TemplateTexture.selectedLogZ(jF);
                            count = count + 1;
                        end
                        
                        % to check whether the extracted feature responses
                        % are correct
                        if abs(S2score - max(S2Map{jT}(:))) > 1e-2
                            disp(S2score);
                            disp(max(S2Map{jT}(:)));
                            pause;
                        end
                        
                        
                        fprintf(fid, '\r\n' );
                        fclose(fid);
                    end

                end
                
		        fprintf(1,' images; takes %.1f sec\n',toc);
            end
        end


		%
		%% testing
        %
        function prepare_svmtest_input(this)
        	nCat = length(this.categories);
            nT = nCat;
            for iCat = 1:nCat
                outputfilepath = sprintf( 'Result/%s/%s/svmtest_run%d.txt', this.categories{iCat}, this.className, this.iRun );
                fid = fopen(outputfilepath,'w');
                fclose(fid);
            end
            clear iCat; clear S2Template; clear S2TemplateTexture

        	% go over categories to fetch templates
        	list_of_S2Template = cell(nT,1);
        	for iT = 1:nCat
        		category = this.categories{iT};
        		load(sprintf('Result/%s/%s/model_run%d.mat',category,this.className,this.iRun),'S2Template','S2TemplateTexture','h');
        		ind = find(S2Template.selectedGain > this.test_gainThres);
				S2Template.selectedRow = S2Template.selectedRow(ind);
				S2Template.selectedCol = S2Template.selectedCol(ind);
				S2Template.selectedOri = S2Template.selectedOri(ind);
				S2Template.selectedScale = S2Template.selectedScale(ind);
				S2Template.selectedLambda = S2Template.selectedLambda(ind);
				S2Template.selectedLogZ = S2Template.selectedLogZ(ind);
        		list_of_S2Template{iT} = S2Template;

				ind = find(S2TemplateTexture.selectedGain > this.test_gainThres);
				S2TemplateTexture.selectedRow = S2TemplateTexture.selectedRow(ind);
				S2TemplateTexture.selectedCol = S2TemplateTexture.selectedCol(ind);
				S2TemplateTexture.selectedOri = S2TemplateTexture.selectedOri(ind);
				S2TemplateTexture.selectedScale = S2TemplateTexture.selectedScale(ind);
				S2TemplateTexture.selectedLambda = S2TemplateTexture.selectedLambda(ind);
				S2TemplateTexture.selectedLogZ = S2TemplateTexture.selectedLogZ(ind);
        		list_of_S2TemplateTexture{iT} = S2TemplateTexture;
        	end
        	disp('templates loaded');
            clear S2Template; clear S2TemplateTexture;

        	% Go over categories to fetch images and compute SUM2 maps
        	for iCategory = 1:nT
        		fprintf(1,'testing on %s: ',this.categories{iCategory});
        		tic
		    	file0 = sprintf('Split/%s/split%d.mat',this.categories{iCategory},this.iRun);
		        load(file0,'posTestImages');
		        nimage = length(posTestImages);
		        scores = cell(nimage,1);

		        for i = 1:nimage
		        	for kk = 1:length(num2str(i-1))
		        		if i == 1
		        		else
		        			fprintf('\b');
		        		end
		        	end
		        	fprintf(1,'%d',i);
				    % for each image, test all templates on it
				    load(sprintf('Feature/%s/%s/%s.mat',...
		                this.featureVersion,this.categories{iCategory},posTestImages{i}),...
                        'S1LocalMean','M1LocalNorm','RANGE');
					S1LocalMean = convert2float(S1LocalMean,RANGE);
					M1 = convert2float(M1LocalNorm,RANGE);

		            % - compute SUM2 maps

                    subsampleS2 = 5;
                    S2MapSketch = mexc_ComputeSUM2( this.norient, M1, list_of_S2Template, subsampleS2 );
                    S2MapTexture = mexc_ComputeSUM2( this.norient, S1LocalMean, list_of_S2TemplateTexture, subsampleS2 );
                    S2Map = cell(size(S2MapSketch));
                    for ii = 1:numel(S2MapSketch)
                        S2Map{ii} = S2MapSketch{ii} + S2MapTexture{ii};
                    end
					
                    tmp = zeros(1,nT,'single');
                    for jT = 1:nCat
                        S2Template = list_of_S2Template{jT};
                        S2TemplateTexture = list_of_S2TemplateTexture{jT};
                        outputfilepath = sprintf( 'Result/%s/%s/svmtest_run%d.txt', this.categories{jT}, this.className, this.iRun );
                        fid = fopen(outputfilepath,'a');
                        
                        [Max2 bestInd] = max(S2Map{jT}(:));
                        tmp(jT) = Max2;
                        col_translation = ceil( bestInd / size(S2Map{1},1) );
                        row_translation = bestInd - (col_translation-1) * size(S2Map{1},1);
                        
                        col_translation = (col_translation - 1) * subsampleS2;
                        row_translation = (row_translation - 1) * subsampleS2;
                        
                        % output ground truth
                        if jT == iCategory
                            fprintf(fid, '%d ', 1);
                        else
                            fprintf(fid, '%d ', -1);
                        end
                        
                        % output sketch features
                        responses_sketch = mexc_ExtractSelectedFeatureResponse( this.norient, M1, list_of_S2Template, subsampleS2, S2Map, outputfilepath );
                        responses_tex = mexc_ExtractSelectedFeatureResponse( this.norient, S1LocalMean, list_of_S2TemplateTexture, subsampleS2, S2Map, outputfilepath );
                        
                        count = 1;
                        
                        S2score = 0;
                        for jF = 1:length(S2Template.selectedRow)
                            val = responses_sketch{jT}(jF);
                            fprintf(fid, '%d:%.5f ', count, val );
                            S2score = S2score + val * S2Template.selectedLambda(jF) - S2Template.selectedLogZ(jF);
                            count = count + 1;
                        end

                        
                        for jF = 1:length(S2TemplateTexture.selectedRow)
                            val = responses_tex{jT}(jF);
                            fprintf(fid, '%d:%.5f ', count, val );
                            S2score = S2score + val * S2TemplateTexture.selectedLambda(jF) - S2TemplateTexture.selectedLogZ(jF);
                            count = count + 1;
                        end
                        
                        % to check whether the extracted feature responses
                        % are correct
                        if abs(S2score - max(S2Map{jT}(:))) > 1e-2
                            disp(S2score);
                            disp(max(S2Map{jT}(:)));
                            pause;
                        end
                        
                        fprintf(fid, '\r\n' );
                        fclose(fid);
                    end
                    
                    scores{i} = tmp;
                end
                
                filename = sprintf('Result/%s/%s/out_run%d.mat', this.categories{iCategory},this.className, this.iRun);
                if exist(filename,'file')
                    save(filename,'-append','scores');
                else
                    save(filename,'scores');
                end
		        fprintf(1,' images; takes %.1f sec\n',toc);
            end
        end


		%%
		function perform_svm(this)
        	nCat = length(this.categories);
			
        	% - on training images:

            % ( use SVM-light )
            for iCat = 1:nCat
                modelfile = sprintf('Result/%s/%s/svmmodel_run%d.txt', this.categories{iCat}, this.className, this.iRun);
                examplefile = sprintf('Result/%s/%s/svmtrain_run%d.txt', this.categories{iCat}, this.className, this.iRun);
                system( sprintf('./svm_learn -c 0.001 %s %s',examplefile,modelfile) );
            end

        	% - on testing images:
            svm_prediction = [];
            for iCat = 1:nCat
                outfile = sprintf('Result/%s/%s/svmout_run%d.txt',this.categories{iCat}, this.className, this.iRun);
                examplefile = sprintf('Result/%s/%s/svmtest_run%d.txt',this.categories{iCat}, this.className, this.iRun);
                modelfile = sprintf('Result/%s/%s/svmmodel_run%d.txt', this.categories{iCat}, this.className, this.iRun);
                system( sprintf('./svm_classify %s %s %s',examplefile,modelfile,outfile) );
                
                % read the outfile
                fid = fopen(outfile,'r');
                predicted = textscan(fid,'%.6f %*[^\n]');
                fclose(fid);
                predicted = predicted{1};
                svm_prediction = [svm_prediction reshape(predicted,length(predicted),1)];
            end
            
            save(sprintf('Result/%s/%s/svmresult_run%d.mat', 'summary', this.className, this.iRun),...
        		'svm_prediction');
            
		end
		
    end
end




