/* mex-C: 
 * compute SUM2 maps for a *single* image
 *
 * Usage:
 *    responses = mexc_ExtractSelectedFeatureResponse( nGaborOri, M1Map, S2Template, subsampleS2, S2Map );
 *
 * S2Template is a matlab struct with the following fields:
 *	selectedRow: single array
 *	selectedCol: single array
 *	selectedOri: single array
 *	selectedScale: single array
 *	selectedLambda: single array
 *	selectedLogZ: single array
 *	
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mex.h"        /* the algorithm is connect to matlab */
#include "math.h"
#include "matrix.h"
#define ABS(x) ((x)>0? (x):(-(x)))
#define MAX(x, y) ((x)>(y)? (x):(y))
#define MIN(x, y) ((x)<(y)? (x):(y))

/* variable declaration */
const float **M1Map;            /* MAX 1 maps */
const float** S2Map;            /* SUM2 maps */
const mxArray* S2Template;      /* SUM2 templates */
int nT;                         /* number of S2 templates */
int* nElement;                  /* number of elements for each template */
int subsampleS2;                /* step length when scanning the SUM2 template over MAX1 map */
int heightS2Map, widthS2Map;    /* (down-sampled) size of SUM2 maps */
int heightM1Map, widthM1Map;    /* size of MAX1 maps */
float** responses;				/* selected feature responses for each template at the MAP position */
int nGaborScale;
int nGaborOri;
char* outputpath;

void compute()
{
    int iT, iF;
    const mxArray* pA, *pA2;
    
    /* Note: we assume that iOri and iS starts from 0. */
    /* Note: we also assume that iRow and iCol is relative to (0,0). */
    int iRowS2, iColS2;
    int iOriM1, iRowM1, iColM1, iScaleM1;
    const float *selectedRow, *selectedCol;
    const float *selectedOri, *selectedScale;
    const float *selectedLambda, *selectedLogZ;
    int rowOffset, colOffset;
    mxClassID datatype;

    heightS2Map = (int)floor((double)heightM1Map / (double)subsampleS2);
    rowOffset = 0;
    widthS2Map = (int)floor((double)widthM1Map / (double)subsampleS2);
    colOffset = 0;
    
    /*
     * compute number of elements for the templates
     */
    nElement = (int*)mxCalloc( nT, sizeof(*nElement) );
    for( iT = 0; iT < nT; ++iT )
    {
        pA = mxGetCell( S2Template, iT );
        pA = mxGetField( pA, 0, "selectedRow" );
        if ( pA == NULL )
		{
			mexErrMsgTxt( "no such field [selectedRow] in struct" );
		}
        nElement[iT] = mxGetM( pA ) * mxGetN( pA );
    }
    
    /*
     * allocate space for responses
     */
    responses = (float**)mxCalloc( nT, sizeof(*responses) );
    for( iT = 0; iT < nT; ++iT )
    {
    	responses[iT] = (float*)mxCalloc( nElement[iT], sizeof(**responses) );
    }
    
    /* About the visiting order in the FOR loop:
     *      The scan over M1 map positions should be inner-most.
     */
    for( iT = 0; iT < nT; ++iT )
    {   
    
    	//FILE* myfile;
  		//myfile = fopen( outputpath, "a" );
        
        pA = mxGetCell( S2Template, iT );
		pA2 = mxGetField( pA, 0, "selectedRow" );
		if ( pA2 == NULL )
		{
			mexErrMsgTxt( "no such field in struct" );
		}
        selectedRow = (const float*)mxGetPr(pA2);
		pA2 = mxGetField( pA, 0, "selectedCol" );
		if ( pA2 == NULL )
		{
			mexErrMsgTxt( "no such field in struct" );
		}
        selectedCol = (const float*)mxGetPr(pA2);
        pA2 = ( mxGetField( pA, 0, "selectedOri" ) );
        if ( pA2 == NULL )
		{
			mexErrMsgTxt( "no such field in struct" );
		}
        selectedOri = (const float*)mxGetPr(pA2);
        pA2 = ( mxGetField( pA, 0, "selectedScale" ) );
        if ( pA2 == NULL )
		{
			mexErrMsgTxt( "no such field in struct" );
		}
		selectedScale = (const float*)mxGetPr(pA2);

        /* find the MAP position */
        float maxVal = S2Map[iT][0];
        int bestCol = 0; int bestRow = 0;
        for( int c = 0; c < widthS2Map; ++c )
        {
        	for( int r = 0; r < heightS2Map; ++r )
        	{
        		int ind = r + c * heightS2Map;
        		if( S2Map[iT][ind] > maxVal )
        		{
        			maxVal = S2Map[iT][ind];
        			bestCol = c; bestRow = r;
        		}
        	}
        }
        
        // for debug:
        /* mexPrintf( "found MAP position \n" );
        mexEvalString( "drawnow" ); */
        
        /* extract feature responses one by one */
        for( iF = 0; iF < nElement[iT]; ++iF )
        {
            iOriM1 = (int)selectedOri[iF];
            iScaleM1 = (int)selectedScale[iF];
            iColS2 = bestCol; iRowS2 = bestRow;
            iColM1 = (int)(colOffset + iColS2 * subsampleS2 + selectedCol[iF]);
            iRowM1 = (int)(rowOffset + iRowS2 * subsampleS2 + selectedRow[iF]);
            if( iRowM1 < 0 || iRowM1 >= heightM1Map || iColM1 < 0 || iColM1 >= widthM1Map )
            {	// out of boundary
            	responses[iT][iF] = 0;
            }
            else
            {
            	responses[iT][iF] = M1Map[iOriM1+iScaleM1*nGaborOri][iRowM1+iColM1*heightM1Map];
            }
            //fprintf( myfile, "%.6f ", responses[iT][iF] );
        }
        
        //fclose( myfile );
    }
}

/* mex function is used to pass on the pointers and scalars from matlab, 
   so that heavy computation can be done by C, which puts the results into 
   some of the pointers. After that, matlab can then use these results. 
   
   So matlab is very much like a managing platform for organizing the 
   experiments, and mex C is like a work enginee for fast computation. */

void mexFunction(int nlhs, mxArray *plhs[], 
                 int nrhs, const mxArray *prhs[])                
{
    int ind, i, x, y, o, dataDim, j, bytes_to_copy, nGaborFilter;
    const mxArray *f;
    const mxArray *pAM1Map;
	mxArray *pA;
    mwSize ndim;
    const mwSize* dims;
    mwSize dimsOutput[2];
    void* start_of_pr;
    mxClassID datatype;
    int iT;

	/*
	 * input variable 0: nGaborOri
	 */
	nGaborOri = (int)mxGetScalar(prhs[0]);
 
    /*
	 * input variable 1: M1 maps
	 */
    pAM1Map = prhs[1];
    dims = mxGetDimensions(pAM1Map);
    nGaborFilter = dims[0] * dims[1];
    nGaborScale = nGaborFilter / nGaborOri;
 
    M1Map = (const float**)mxCalloc( nGaborFilter, sizeof(*M1Map) );   /* MAX1 maps */
    for (i=0; i<nGaborFilter; ++i)
    {
        f = mxGetCell(pAM1Map, i);
        datatype = mxGetClassID(f);
        if (datatype != mxSINGLE_CLASS)
            mexErrMsgTxt("warning !! single precision required.");
        M1Map[i] = (const float*)mxGetPr(f);    /* get the pointer to cell content */
        heightM1Map = mxGetM(f);    /* overwriting is ok, since it is constant */
        widthM1Map = mxGetN(f);
    }

    /*
     * input variable 2: S2 templates
     */
    S2Template = prhs[2];
    nT = mxGetM(S2Template) * mxGetN(S2Template);
    
    /*
     * input variable 3: subsampleS2
     */
    subsampleS2 = (int)mxGetScalar(prhs[3]);
    
    /*
     * input variable 4: S2Map
     */
    S2Map = (const float**)mxCalloc( nT, sizeof(*S2Map) );   /* SUM2 maps */
    for (i=0; i<nT; ++i)
    {
        f = mxGetCell(prhs[4], i);
        datatype = mxGetClassID(f);
        if (datatype != mxSINGLE_CLASS)
            mexErrMsgTxt("warning !! single precision required.");
        S2Map[i] = (const float*)mxGetPr(f);    /* get the pointer to cell content */
    }
    
    /*
     * input variable 5: output file path
     */
	const mxArray* pAString = prhs[5];
    unsigned int yLength = mxGetN(pAString)+1;
    outputpath = (char*)mxCalloc(yLength, sizeof(char));
    mxGetString(pAString,outputpath,yLength);

    compute();
    
    /* =============================================
     * Handle output variables.
     * ============================================= 
     */
    /*
     * output variable 0: responses
     */
    dimsOutput[0] = nT; dimsOutput[1] = 1;
	plhs[0] = mxCreateCellArray( 2, dimsOutput );
    
    for( iT = 0; iT < nT; ++iT )
    {
    	dimsOutput[0] = nElement[iT]; dimsOutput[1] = 1;
        pA = mxCreateNumericArray( 2, dimsOutput, mxSINGLE_CLASS, mxREAL );
        /* populate the real part of the created array */
        start_of_pr = (float*)mxGetData(pA);
        bytes_to_copy = dimsOutput[0] * dimsOutput[1] * mxGetElementSize(pA);
        memcpy( start_of_pr, responses[iT], bytes_to_copy );
        mxSetCell( plhs[0], iT, pA );
    }
}

